using System.Numerics;
using Raylib_cs;

namespace Tiles;

public class Game
{
    public static Game Instance { get; private set; }
    
    public TileManager TileManager { get; private set; }
    public Player Player { get; private set; }
    public CameraController CameraController { get; private set; }
    public World World { get; private set; }
    public DebugUI DebugUi { get; private set; }

    public int ScreenWidth { get; private set; }
    public int ScreenHeight { get; private set; }
    
    public Game(int screenWidth, int screenHeight)
    {
        ScreenWidth = screenWidth;
        ScreenHeight = screenHeight;
        
        Instance = this;
        InitializeRaylib();
        Init();

        while (!Raylib.WindowShouldClose())
        {
            Draw();
            Update();
        }
    }

    private void InitializeRaylib()
    {
        Raylib.InitWindow(ScreenWidth, ScreenHeight, "Tiles");
        Raylib.SetTargetFPS(60);
    }

    private void Init()
    {
        TileManager = new TileManager();
        
        World = new World();
        
        Player = new Player("Player")
        {
            Position = World.GetGlobalPositionFromGrid(500, 200)
        };
        
        CameraController = new CameraController();
        CameraController.SetCameraTarget(Player.Position);
        CameraController.SetCameraOffset(new Vector2(ScreenWidth / 2, ScreenHeight / 2));
        CameraController.SetCameraRotation(0f);
        CameraController.SetCameraZoom(4f);
        
        DebugUi = new DebugUI();
        
        World.GenerateWorld();
    }

    private void Update()
    {
        CameraController.SetCameraTarget(Player.Position);
        Player.Update();
    }

    private void Draw()
    {
        Raylib.BeginDrawing();
        Raylib.ClearBackground(new Color(20, 30, 100, 255));
        Raylib.BeginMode2D(CameraController.Camera); 
        
        World.Draw();
        Player.Draw();
        DebugUi.Draw();
           
        Raylib.EndMode2D();
        Raylib.EndDrawing();
    }


    

    
    

    
    
    
    
    
}